// *****************************************************************************
// Module: DCC_comm.cpp 
// Type: C++/C dll
// Platform: Intel/Win95/ms visual c++ 6.0
// Authors: D C Pankratz, Jeremy Vosters
// Date Created: April 27, 2001
//
// Purpose: Provide an API for Wangrow's System One Command Station to 
//			operate trains with decoders using Digital Command Control (DCC)
//          as specified by the NMRA standards and recommended practices.
//
// Description: Provides serial comm I/O functions on an Intel 1996 computer.
//				Provides Wangrow System One Command Station (1999) functions.
//				Specifically, the DCC_API consists of:
//					DCC_motor(args); control the speed and direction of a train
//					DCC_function(args); control up to 4 optional fcns on a train
//					DCC_brake(arg); slows a train to a stop
//					DCC_kill(); kills the power on the main track
//					DCC_reset(); re-establishes power to the main track
//
// Modified: (DATE/AUTHOR/REASON/ACTIVITY)
//
// ***********************************************************************************				

//#include "DCC_comm.h



void close_comm_port(HANDLE hcomm){
// releases the comm port
    CloseHandle(hcomm);
	return;	
}


int open_comm_port(char commnumber[5], HANDLE& hcomm){
// open a commport that is sent and return its handle
	hcomm = CreateFile(commnumber,
		       GENERIC_READ | GENERIC_WRITE,
			   0, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL,0);

	if (hcomm == INVALID_HANDLE_VALUE) return 0;
	return 1;
} // open_comm_port


int set_comm_state(HANDLE hcomm,DCB& dcb, int baudrate, int bytesize, 
				    int parity, int stopbits){
// sets the comm state in a large struct called DCB.
	COMMTIMEOUTS timeouts;
    BOOL fsuccess;
    fsuccess = GetCommState(hcomm, &dcb);
	if (!fsuccess) return 0;
	dcb.BaudRate = baudrate;
	dcb.ByteSize = bytesize;
	dcb.Parity   = parity;
	dcb.StopBits = stopbits;
	fsuccess = SetCommState(hcomm, &dcb);
	if (!fsuccess) return 0;
		
	// Set the readintervaltimeout so that ReadFile returns
	// immediately even if the comm buffer is empty.
	timeouts.ReadIntervalTimeout = MAXDWORD; //immediate read
	// set to 0 to wait forever, 1000 for 1 sec between chars.
    timeouts.ReadTotalTimeoutMultiplier  = 0;
    timeouts.ReadTotalTimeoutConstant    = 0;
	timeouts.WriteTotalTimeoutMultiplier = 0;
	timeouts.WriteTotalTimeoutConstant    = 0;
	fsuccess = SetCommTimeouts(hcomm,&timeouts);
	if (!fsuccess) return 0;
	return 1;
} //set_comm_state



int writecomm(HANDLE hcomm, char command[]){
/* writes a block of chars (bytes) to the port */
	DWORD numWrite;
	int success = WriteFile(hcomm,command,strlen(command),&numWrite,0);
    if (success)
		return 1;
	return 0;
} // writecomm



int readcomm(HANDLE DCC_comm, char command[]){
/* reads a block of chars (bytes) from the port */
/* read is immediate depending on function SetCommtimeouts()
/* returns number of bytes read 0..100 */
/* returns -1 on error */
	DWORD numRead;
	int success = ReadFile(DCC_comm,command,100,&numRead,0);
    if (!success)
		return -1;
	
	return (int)numRead;
} // readcomm




int immed_readcomm(HANDLE DCC_comm, char response[]){
// Reads from the port. Timesout after 25*cnt millesec..MAX = 500ms
// --- only used if comm state is set for immediate read!!!!!!!
// Returns the block of bytes read: response
// Returns number of bytes read (return)
// Returns 0 if timeout.
// Returns -1 if problems with comm port.
	  int n, cnt;
      n= 0; cnt= 0;
      for (cnt=0; cnt<50;cnt++) { // up to 0.5 sec
		//cerr << "cnt = " << cnt << endl;
		n = readcomm(DCC_comm,response);
		if (n== -1) return n;
		if (n>0) {
			//cerr << "char from station " << response[0] << endl;return n;
		}
   	    Sleep(25);
	  }
	  return 0;
} // immed_readcomm



//**********************************************************************************
//*********************************************************************************


int DCC_open( char comportnumber[], HANDLE& DCC_comm)  {
// Opens a serial port to communicate with the DCC command station 
// and establishes the required protocol: 9600,8,0,1
//
// input:  comportnumber - str "COM1" or "COM2" or ..
// output: DCC_comm - handle to the comm port
// return 0 if error
// return 1 if open and protocol accepted.
//****************************************************************************88
	int success;   
	DCB dcb;
    success = open_comm_port(comportnumber, DCC_comm);
    if (!success) return 0;
    success = set_comm_state(DCC_comm, dcb, 9600, 8, NOPARITY, ONESTOPBIT);
    if (!success) return 0;
    return 1;
} // DCC_open


void DCC_close(HANDLE DCC_comm){
// releases the connection
    close_comm_port(DCC_comm);
	return;	
}


int DCC_motor(HANDLE DCC_comm, int id, int dir, int speed) {
// A System One binary command to modify a motor on a train decoder.
// The motor cmd will be buffered in the command station and 
// continually sent every 25 mu sec onto the track until another
// DCC command is sent.
// Input: id - train's long address 0 - 255
//		  dir - new direction: 1 == forward, 0 ==reverse
//  	  speed - new speed: 0-28
//
// Return: 1 -  command station took the instruction-delivery not guaranteed
//         0 -  command station did not receive the command
//         -1 - command station had problems with id
//         -2 - direction out of range
//         -3 - speed out of range
//************************************************************************************
	char cmd[10], c;
	zerobuffer(cmd, sizeof(cmd));
	cmd[0] = (char) 0xA3; // opcode for motor cmd
	//cerr << "id:" << id << endl;
	cmd[1] = (char) id;
	//cerr << "motor+id:";
	//displaycommandhex(cmd, strlen(cmd));

	
	if (speed < 0 || speed > 28)  return -3; 
	if (speed >= 1 || speed <= 14) speed++;  // DCC reserved 0 = brake and 1 estop
	else if ( speed >= 15)  speed += 3; // DCC reserved 16 = brake and 17 = estop
	// 0 == 16: brake(chip can be prototyped for inertia)
	// 1 == 17: emergency stop - cut the motor. 
    //printf("speed = %x\n", speed);	
	//printf("dir = %x\n", dir);	
	// mode byte: 01dsssss where d = dir and s = speed
	if ( dir == 0 || dir == 1) {
		dir = dir << 5; // shift left 5
		cmd[2] = 0x40 | (char)dir | speed; // add in the direction and speed
	}
	else return -2; // direction out of range
	//printf("dir = %x\n", dir);	
	cmd[3] = cmd[1] ^ cmd[2]; // chksum is the xor
	if (writecomm(DCC_comm, cmd) ==0) return 0;
	//cerr << "full cmd:";
	//displaycommandhex(cmd, strlen(cmd));
	
	if (!(immed_readcomm(DCC_comm, &c))) return 0;
	if (c=='1') return -1;
	return 1;
} // DCC_motor



int DCC_Estop(HANDLE DCC_comm, int id) {
// A System One binary command to cut the power on a train motor.
// The train will stop as soon as possible.
// Choosing the DCC_motor() fucntion with speed = 0 will cause 
// the motor will brake according to the inertia programmed into the decoder.
// 
// Note: there is also a DCC_kill() function which will cut the power 
// to the entire track layout.
//************************************************************************************
	char cmd[10], c;
	zerobuffer(cmd, sizeof(cmd));
	cmd[0] = (char) 0xA3; // opcode for motor cmd
	cmd[1] = (char) id;   // train id
	cmd[2] = (char) 1;	  // emergency stop
	cmd[3] = cmd[1] ^ cmd[2];		// chksum
	if (writecomm(DCC_comm, cmd) ==0) return 0;
	
	if (!(immed_readcomm(DCC_comm, &c))) return 0;
	if (c=='1') return -1;
	return 1;
} // DCC_Estop




int DCC_function(HANDLE DCC_comm, int train_id, int &fcn_state, 
				 int fcn_id, int value) {
// A System One binary command to modify one of the first 4 functions
// supported by a decoder. Usually, 0==headlight, 1==taillight or
// marslights.
//
// The function cmd will be only sent a specific number of times == DCC_SENDCNT.
//	
// Input: id         - train's long address 0 - 255
//		  fcn_state  - current state of the 5 fcns
//		  fcn_id - function id 0-4 to change
//  	  value  - new value 0 == off, 1 == on
//
// Return: fcn_state - updates state variable for the functions of this train
//		   1  - command station took the instruction-delivery not guaranteed
//         0  - command station did not receive the command
//         -1 - command station had problems with train_id
//         -2 - fcn_id out of range
//         -3 - value out of range
//************************************************************************************
	const int DCC_SENDCNT = 6;
	char cmd[10], c;
	int bit;
		
	zerobuffer(cmd, sizeof(cmd));
	cmd[0] = (char) 0x93; // opcode for group 1 function cmd
	
	if (fcn_id <0 || fcn_id >4) return -2;
	if (value < 0 || value > 1) return -3;
	
	cmd[1] = (char) DCC_SENDCNT; //number of times to send the command
	
	value = value << fcn_id;  // move the new bit into position
	bit = 1;
	bit = bit << fcn_id;
	fcn_state = fcn_state & bit; // clear bit in the current state
	fcn_state = fcn_state | value; // insert the new value
	fcn_state = fcn_state & 0x1F;  // mask out first 3 bits to be sure
	cmd[2] = 0x80 | fcn_state;
	cmd[3] = cmd[1] ^ cmd[2]; // chksum is the xor
	if (writecomm(DCC_comm, cmd) ==0) return 0;
	
//	cerr << "full cmd:";
//	displaycommandhex(cmd, strlen(cmd));
	
	if (!(immed_readcomm(DCC_comm, &c))) return 0;
	if (c=='1') return -1; //************************************8check on this
	return 1;
} // DCC_function



int DCC_kill(HANDLE DCC_comm) {
// A System One binary command to cut the power on the main track. 
// Used as an emergency BRAKE for ALL trains.
// All train settings are zeroed by the hardware (DCC_motor and DCC_function).
// Must call DCC_reset to re-establish power to the main track.
//
// Return: 1 -  command station took the instruction.
//         0 -  command station did not receive the command
// ************************************************************************************
	char cmd[4], c;
	zerobuffer(cmd, sizeof(cmd));
	cmd[0] = (char) 0x8B; // opcode for kill cmd
	
	if (writecomm(DCC_comm, cmd) ==0) return 0;
	
	if (!(immed_readcomm(DCC_comm, &c))) return 0;
	return 1;
} // DCC_kill


int DCC_reset(HANDLE DCC_comm) {
// A System One binary command to restore the power on the main track. 
// Used after the DCC_kill() is called.
// All train settings are zeroed by the hardware (DCC_motor and DCC_function).
// Must call DCC_motor() and/or DCC_function() to reset trains.
//
// Return: 1 -  command station took the instruction.
//         0 -  command station did not receive the command
// ************************************************************************************
	char cmd[4], c;
	zerobuffer(cmd, sizeof(cmd));
	cmd[0] = (char) 0x89; // opcode for reset cmd
	
	if (writecomm(DCC_comm, cmd) ==0) return 0;
	
	if (!(immed_readcomm(DCC_comm, &c))) return 0;
	return 1;
} // DCC_reset


bool DCC_testcommands(void) {
// UNDER CONSTRUCTION
// This fcn should walk the user through a series of tests
// to check out the connections from the comm port to the cmd station,
// from the cmd station to a decoder, and basic functions on a decoder.
// **********************************************************************************
//	cerr << "test commands\n";
	return 0;
}


bool DCC_command( char cmd_str[]) {
// UNDER CONSTRUCTION
// This fcn would send ANY command to the cmd station.
// Issues -- should it be the binary or ascii format??
// Should it compute the cksum?
// **********************************************************************************
//	cerr << "\n DCC command string\n";
//	cerr << cmd_str << endl;
	return 0;
}



	
void zerobuffer(char * buf, int n)  {
// makes cstrings as pure as Christmas snow.
	int i;
	for (i=0;i<n; i++)
		buf[i] = 0;
	return;
}

void displaycommandhex(char * buf, int n) {
// debugging tool for system one DCC format - cstring of binary:
// cmd data ... chksum
   int i;
  // cerr << "command string:";
   //for (i=0;i<n;i++)
	  //printf("%x ", (byte)buf[i]);
   //cout << '\n';
   return;
}

// END DCC_comm
// ***************************************************************************




