// *****************************************************************************
// Module: DCC_comm.h 
// Type: C++/C dll
// Platform: Intel/Win95/ms visual c++ 6.0
// Authors: D C Pankratz, Jeremy Vosters
// Date Created: April 27, 2001
//
// Purpose: Provide an API for Wangrow's System One Command Station to 
//			operate trains with decoders using Digital Command Control (DCC)
//          as specified by the NMRA standards and recommended practices.
//
// Description: Provides serial comm I/O functions on an Intel 1996 computer.
//				Provides Wangrow System One Command Station (1999) functions.
//				Specifically, the DCC_API consists of:
//					DCC_motor(args); control the speed and direction of a train
//					DCC_function(args); control up to 4 optional fcns on a train
//					DCC_Estop(arg); slows a train to a stop
//					DCC_kill(); kills the power on the main track
//					DCC_reset(); re-establishes power to the main track
//
// Modified: (DATE/AUTHOR/REASON/ACTIVITY)
//
// ***********************************************************************************				




int DCC_open( char comportnumber[], unsigned long& DCC_comm);
// Opens a serial port to communicate with the DCC command station 
// and establishes the required protocol: 9600,8,0,1
//
// input:  comportnumber - str "COM1" or "COM2" or ..
// output: DCC_comm - handle to the comm port
// return 0 if error
// return 1 if open and protocol accepted.
//****************************************************************************88


void DCC_close(unsigned long DCC_comm);
// releases the comm port
//************************************************************************************



int DCC_motor(unsigned long DCC_comm, int id, int dir, int speed);
// A System One binary command to modify a motor on a train decoder.
// The motor cmd will be buffered in the command station and 
// continually sent every 25 mu sec onto the track until another
// DCC command is sent.
// Input: id - train's long address 0 - 255
//		  dir - new direction: 1 == forward, 0 ==reverse
//  	  speed - new speed: 0-28
//
// Return: 1 -  command station took the instruction-delivery not guaranteed
//         0 -  command station did not receive the command
//         -1 - command station had problems with id
//         -2 - direction out of range
//         -3 - speed out of range
//
//************************************************************************************


int DCC_Estop(unsigned long DCC_comm, int id);
// A System One binary command to cut the power on a train motor.
// The train will stop as soon as possible.
// Choosing the DCC_motor() fucntion with speed = 0 will cause 
// the motor will brake according to the inertia programmed into the decoder.
// 
// Note: there is also a DCC_kill() function which will cut the power 
// to the entire track layout.
//************************************************************************************



//int DCC_function(HANDLE DCC_comm, int train_id, int &fcn_state, 
//				 int fcn_id, int value);
// A System One binary command to modify one of the first 4 functions
// supported by a decoder. Usually, 0==headlight, 1==taillight or
// marslights.
//
// The function cmd will be only sent a specific number of times == DCC_SENDCNT.
//	
// Input: id         - train's long address 0 - 255
//		  fcn_state  - current state of the 5 fcns
//		  fcn_id - function id 0-4 to change
//  	  value  - new value 0 == off, 1 == on
//
// Return: fcn_state - updates state variable for the functions of this train
//		   1  - command station took the instruction-delivery not guaranteed
//         0  - command station did not receive the command
//         -1 - command station had problems with train_id
//         -2 - fcn_id out of range
//         -3 - value out of range
//************************************************************************************




int DCC_kill(unsigned long DCC_comm);
// A System One binary command to cut the power on the main track. 
// Used as an emergency BRAKE for ALL trains.
// All train settings are zeroed by the hardware (DCC_motor and DCC_function).
// Must call DCC_reset to re-establish power to the main track.
//
// Return: 1 -  command station took the instruction.
//         0 -  command station did not receive the command
// ************************************************************************************


int DCC_reset(unsigned long DCC_comm);
// A System One binary command to restore the power on the main track. 
// Used after the DCC_kill() is called.
// All train settings are zeroed by the hardware (DCC_motor and DCC_function).
// Must call DCC_motor() and/or DCC_function() to reset trains.
//
// Return: 1 -  command station took the instruction.
//         0 -  command station did not receive the command
// ************************************************************************************


//bool DCC_testcommands(int & testvalue);
// UNDER CONSTRUCTION
// This fcn should walk the user through a series of tests
// to check out the connections from the comm port to the cmd station,
// from the cmd station to a decoder, and basic functions on a decoder.
// **********************************************************************************


//bool DCC_command( char cmd_str[]);
// UNDER CONSTRUCTION
// This fcn would send ANY command to the cmd station.
// Issues -- should it be the binary or ascii format??
// Should it compute the cksum?
// **********************************************************************************

// *********************************************************************************
// *********************************************************************************

// END DCC_comm.h
// ***********************************************************************************



